@@ -1,19 +1,19 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2006-2019 sqlmap developers (http://sqlmap.org/)
+Copyright (c) 2006-2021 sqlmap developers (http://sqlmap.org/)
 See the file 'LICENSE' for copying permission
 """
 
 import copy
 import types
 
-from thirdparty.odict.odict import OrderedDict
+from thirdparty.odict import OrderedDict
+from thirdparty.six.moves import collections_abc as _collections
 
 class AttribDict(dict):
     """
-    This class defines the sqlmap object, inheriting from Python data
-    type dictionary.
+    This class defines the dictionary with added capability to access members as attributes
 
     >>> foo = AttribDict()
     >>> foo.bar = 1
@@ -21,13 +21,14 @@ class AttribDict(dict):
     1
     """
 
-    def __init__(self, indict=None, attribute=None):
+    def __init__(self, indict=None, attribute=None, keycheck=True):
         if indict is None:
             indict = {}
 
         # Set any attributes here - before initialisation
         # these remain as normal attributes
         self.attribute = attribute
+        self.keycheck = keycheck
         dict.__init__(self, indict)
         self.__initialised = True
 
@@ -43,7 +44,10 @@ class AttribDict(dict):
         try:
             return self.__getitem__(item)
         except KeyError:
-            raise AttributeError("unable to access item '%s'" % item)
+            if self.keycheck:
+                raise AttributeError("unable to access item '%s'" % item)
+            else:
+                return None
 
     def __setattr__(self, item, value):
         """
@@ -109,6 +113,19 @@ class InjectionDict(AttribDict):
 
 # Reference: https://www.kunxi.org/2014/05/lru-cache-in-python
 class LRUDict(object):
+    """
+    This class defines the LRU dictionary
+
+    >>> foo = LRUDict(capacity=2)
+    >>> foo["first"] = 1
+    >>> foo["second"] = 2
+    >>> foo["third"] = 3
+    >>> "first" in foo
+    False
+    >>> "third" in foo
+    True
+    """
+
     def __init__(self, capacity):
         self.capacity = capacity
         self.cache = OrderedDict()
@@ -120,15 +137,12 @@ class LRUDict(object):
         return key in self.cache
 
     def __getitem__(self, key):
-        try:
-            value = self.cache.pop(key)
-            self.cache[key] = value
-            return value
-        except KeyError:
-            return -1
+        value = self.cache.pop(key)
+        self.cache[key] = value
+        return value
 
     def get(self, key):
-        return self.__getitem__(self, key)
+        return self.__getitem__(key)
 
     def __setitem__(self, key, value):
         try:
@@ -143,3 +157,76 @@ class LRUDict(object):
 
     def keys(self):
         return self.cache.keys()
+
+# Reference: https://code.activestate.com/recipes/576694/
+class OrderedSet(_collections.MutableSet):
+    """
+    This class defines the set with ordered (as added) items
+
+    >>> foo = OrderedSet()
+    >>> foo.add(1)
+    >>> foo.add(2)
+    >>> foo.add(3)
+    >>> foo.pop()
+    3
+    >>> foo.pop()
+    2
+    >>> foo.pop()
+    1
+    """
+
+    def __init__(self, iterable=None):
+        self.end = end = []
+        end += [None, end, end]         # sentinel node for doubly linked list
+        self.map = {}                   # key --> [key, prev, next]
+        if iterable is not None:
+            self |= iterable
+
+    def __len__(self):
+        return len(self.map)
+
+    def __contains__(self, key):
+        return key in self.map
+
+    def add(self, value):
+        if value not in self.map:
+            end = self.end
+            curr = end[1]
+            curr[2] = end[1] = self.map[value] = [value, curr, end]
+
+    def discard(self, value):
+        if value in self.map:
+            value, prev, next = self.map.pop(value)
+            prev[2] = next
+            next[1] = prev
+
+    def __iter__(self):
+        end = self.end
+        curr = end[2]
+        while curr is not end:
+            yield curr[0]
+            curr = curr[2]
+
+    def __reversed__(self):
+        end = self.end
+        curr = end[1]
+        while curr is not end:
+            yield curr[0]
+            curr = curr[1]
+
+    def pop(self, last=True):
+        if not self:
+            raise KeyError('set is empty')
+        key = self.end[1][0] if last else self.end[2][0]
+        self.discard(key)
+        return key
+
+    def __repr__(self):
+        if not self:
+            return '%s()' % (self.__class__.__name__,)
+        return '%s(%r)' % (self.__class__.__name__, list(self))
+
+    def __eq__(self, other):
+        if isinstance(other, OrderedSet):
+            return len(self) == len(other) and list(self) == list(other)
+        return set(self) == set(other)
