@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2006-2019 sqlmap developers (http://sqlmap.org/)
+Copyright (c) 2006-2021 sqlmap developers (http://sqlmap.org/)
 See the file 'LICENSE' for copying permission
 """
 
@@ -9,9 +9,15 @@ import os
 import re
 import subprocess
 
+from lib.core.common import openFile
+from lib.core.convert import getText
+
 def getRevisionNumber():
     """
     Returns abbreviated commit hash number as retrieved with "git rev-parse --short HEAD"
+
+    >>> len(getRevisionNumber() or (' ' * 7)) == 7
+    True
     """
 
     retVal = None
@@ -31,12 +37,17 @@ def getRevisionNumber():
 
     while True:
         if filePath and os.path.isfile(filePath):
-            with open(filePath, "r") as f:
-                content = f.read()
+            with openFile(filePath, "r") as f:
+                content = getText(f.read())
                 filePath = None
+
                 if content.startswith("ref: "):
-                    filePath = os.path.join(_, ".git", content.replace("ref: ", "")).strip()
-                else:
+                    try:
+                        filePath = os.path.join(_, ".git", content.replace("ref: ", "")).strip()
+                    except UnicodeError:
+                        pass
+
+                if filePath is None:
                     match = re.match(r"(?i)[0-9a-f]{32}", content)
                     retVal = match.group(0) if match else None
                     break
@@ -44,9 +55,12 @@ def getRevisionNumber():
             break
 
     if not retVal:
-        process = subprocess.Popen("git rev-parse --verify HEAD", shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        stdout, _ = process.communicate()
-        match = re.search(r"(?i)[0-9a-f]{32}", stdout or "")
-        retVal = match.group(0) if match else None
+        try:
+            process = subprocess.Popen("git rev-parse --verify HEAD", shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            stdout, _ = process.communicate()
+            match = re.search(r"(?i)[0-9a-f]{32}", getText(stdout or ""))
+            retVal = match.group(0) if match else None
+        except:
+            pass
 
     return retVal[:7] if retVal else None
