@@ -1,13 +1,15 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2006-2019 sqlmap developers (http://sqlmap.org/)
+Copyright (c) 2006-2021 sqlmap developers (http://sqlmap.org/)
 See the file 'LICENSE' for copying permission
 """
 
 import binascii
 
 from lib.core.common import Backend
+from lib.core.compat import xrange
+from lib.core.convert import getBytes
 from lib.core.data import logger
 from lib.core.exception import SqlmapUnsupportedFeatureException
 from lib.request import inject
@@ -66,7 +68,7 @@ class Takeover(GenericTakeover):
             raise SqlmapUnsupportedFeatureException(errMsg)
 
         shellcodeChar = ""
-        hexStr = binascii.hexlify(self.shellcodeString[:-1])
+        hexStr = binascii.hexlify(getBytes(self.shellcodeString[:-1]))
 
         for hexPair in xrange(0, len(hexStr), 2):
             shellcodeChar += "CHAR(0x%s)+" % hexStr[hexPair:hexPair + 2]
